/* This is a set of Mata functions used in the Model file, and in particular in 
the Gibbs sampler. */

mata:

void RunChecks(real matrix FE, real matrix WWA, real matrix WWB, real matrix Ins, ///
real matrix lnPrice, real matrix Procedure, real matrix NonResp, real matrix RegNum, ///
real matrix Year, real matrix HHI_Thirds, string scalar CovNL, string scalar CovFE, ///
real scalar RdForm){
   	/* if(max(WWB) > 2 | min(WWB) < -2) exit(_error("Extreme value.")) */ /* Want 
   	variables to have roughly same range. Useful when choosing random starting points. 
   	*/
	cmdstring = "reg thpp " + CovNL + " " + CovFE + ", nocons"
	stata(cmdstring) /* Run regression to check for collinearity of RHS variables. */
	if (sum(st_matrix("e(b)") :== 0) != 0) exit(_error("Collinearity."))
	if (missing((FE, WWA, WWB, Ins, lnPrice, Procedure, NonResp, RegNum, Year, /// 
		HHI_Thirds)) != 0) exit(_error("Missing value."))
    if (RdForm == 1) assert(cols(WWA) < cols(WWB))
    else assert(WWA == WWB)
}

/* Declare the data structure. This is an an argument of LogLWrapper(). */
struct DataStruct{
	real scalar TypebP, i1, i2, i3, i4, i5, i6
	real matrix FE, WWA, WWB, Ins, lnPrice, Procedure, NonResp, RegNum, Year, HHI_Thirds
}

/* Define the data structure, given the arguments. */
struct DataStruct scalar CreateDataStruct(real scalar nparbP, real scalar TypebP, ///
real matrix FE, real matrix WWA, real matrix WWB, real matrix Ins, ///
real matrix lnPrice, real matrix Procedure, real matrix NonResp, real matrix RegNum, ///
real matrix Year, real matrix HHI_Thirds){
	struct DataStruct scalar Data
		Data.TypebP = TypebP
		Data.i1 = cols(FE)*5
		Data.i2 = Data.i1 + cols(WWA)*5
		Data.i3 = Data.i2 + cols(Ins)*5
		Data.i4 = Data.i3 + cols(FE)*4
		Data.i5 = Data.i4 + cols(WWB)*4
		Data.i6 = Data.i5 + nparbP
		Data.FE = FE
		Data.WWA = WWA
		Data.WWB = WWB
		Data.Ins = Ins
		Data.lnPrice = lnPrice
		Data.Procedure = Procedure
		Data.NonResp = NonResp
		Data.RegNum = RegNum
		Data.Year = Year
		Data.HHI_Thirds = HHI_Thirds
	return(Data)	
}


/* The purpose of this function is to calculate the log-likelihood (by calling LogL())
but with only three arguments. xx contains all the parameters. The DataStruct structure
called Data contains all the data needed for the log-likelihood evaluation. yy contains
the output. This function is in a form Mata can numerically differentiate, unlike
LogL. If ReturnMean is 1 then yy contains the mean log likelihood, otherwise it 
is a vector containing the likelihood for each individual. */
void LogLWrapper(real matrix xx, struct DataStruct scalar Data, ///
real scalar ReturnMean, real matrix yy){
	aFE = ToMatA(xx[1 .. Data.i1])
	aW = ToMatA(xx[Data.i1 + 1 .. Data.i2])
	aIns = ToMatA(xx[Data.i2 + 1 .. Data.i3])
	bFE = ToMatB(xx[Data.i3 + 1 .. Data.i4])
	bW = ToMatB(xx[Data.i4 + 1 .. Data.i5])
	bP = xx[Data.i5 + 1 .. Data.i6]
	Sig = invvech(xx[Data.i6 + 1 .. cols(xx)]')
 	yy = LogL(aFE, aW, aIns, bFE, bW, bP, Sig, Data.TypebP, Data.FE, Data.WWA, ///
		Data.WWB, Data.Ins, Data.lnPrice, Data.Procedure, Data.NonResp, Data.RegNum, ///
 		Data.Year, Data.HHI_Thirds, ReturnMean, ResetSeed = 1, 9494)
}


/* This replaces SimLikelihood(). Uses GHK rather than crude frequency simulation. */
real matrix LogL(real matrix aFE, real matrix aW, real matrix aIns, real matrix bFE, ///
real matrix bW, real matrix bP, real matrix Sig, real scalar TypebP, ///
real matrix FE, real matrix WWA, real matrix WWB, real matrix Ins, ///
real matrix lnPrice, real matrix Procedure, real matrix NonResp, real matrix RegNum, ///
real matrix Year, real matrix HHI_Thirds, real scalar ReturnMean, ///
real scalar ResetSeed, | real scalar seed){
	if (ResetSeed == 1) rseed(seed) /* If calculating derivatives, want to use the 
	same seed so that the same random numbers are generated for each function call. */
	cmlnPP = FE*aFE + WWA*aW + Ins*aIns
	PredUtil = FE*bFE + WWB*bW + cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, Year, /// 
		HHI_Thirds)
	/* Find predicted utility from choices 2 to 5, and the variance of the utility errors 
	conditional on observing a price error, for each price error. */	
	coefS1 = Sig[6..9, 1]*(Sig[1, 1]^-1)
	coefS2 = Sig[6..9, 2]*(Sig[2, 2]^-1)
	coefS3 = Sig[6..9, 3]*(Sig[3, 3]^-1)
	coefS4 = Sig[6..9, 4]*(Sig[4, 4]^-1)
	coefS5 = Sig[6..9, 5]*(Sig[5, 5]^-1)
	coefS = (coefS1, coefS2, coefS3, coefS4, coefS5) 
	condS1 = Sig[6..9, 6..9] - coefS1*Sig[1, 6..9]
	condS2 = Sig[6..9, 6..9] - coefS2*Sig[2, 6..9]
	condS3 = Sig[6..9, 6..9] - coefS3*Sig[3, 6..9]
	condS4 = Sig[6..9, 6..9] - coefS4*Sig[4, 6..9]
	condS5 = Sig[6..9, 6..9] - coefS5*Sig[5, 6..9]
	condS = (condS1, condS2, condS3, condS4, condS5) 

	/* Define the relevant transformations. */
	c1 = J(4,1,-1)
	c2 = (0\1\0\0)
	c3 = (0\0\1\0)
	c4 = (0\0\0\1)
	T1 = I(4)
	T2 = (c1,c2,c3,c4)
	T3 = (c2,c1,c3,c4)
	T4 = (c2,c3,c1,c4)
	T5 = (c2,c3,c4,c1)
	TT = (T1,T2,T3,T4,T5)
	nobs = rows(WWA)
	ChoiceProb = J(nobs, 1, .)
	LogL = J(nobs, 1, .)
	S = ghkfast_init(nobs, 50, 4, "ghalton") /* Second argument is number of draws. */
	ghkfast_init_pivot(S, 0) /* Turn off pivoting. Mata manual says this is important to 
		avoid discontinuities when computing numerical second-order derivatives. */
	for (ii = 1; ii <= nobs; ii++){
		Observed_uuii = lnPrice[ii] - FE[ii, .]*aFE[., Procedure[ii]] - /// 
			WWA[ii, .]*aW[., Procedure[ii]] - Ins[ii, .]*aIns[., Procedure[ii]]
		coefSProcii = coefS[., Procedure[ii]]
		condSProcii = ind(condS, Procedure[ii])
		TProcii = ind(TT, Procedure[ii])
		Cm = coefSProcii*Observed_uuii	
		PredUtilPlusCmii = PredUtil[ii, .]' + Cm
		ChoiceProb[ii] = ghkfast_i(S, -(TProcii*PredUtilPlusCmii)', ///
			TProcii*condSProcii*TProcii', 1)
		LogL[ii] = log(ChoiceProb[ii]) + ///
			log(normalden(Observed_uuii, 0, Sig[Procedure[ii], Procedure[ii]]^0.5))
	}	
	if (ReturnMean == 1) return(mean(LogL))	
	else return(LogL')
}

real matrix ind(real matrix TT, real scalar ind){
	return(TT[., 4*(ind - 1) + 1 .. 4*ind])
}

/* These are the indices of the other errors, e.g. im1 omits the first error. */
real matrix defineindmat(){
	im1 = (2,3,4,5,6,7,8,9)
	im2 = (1,3,4,5,6,7,8,9)
	im3 = (1,2,4,5,6,7,8,9)
	im4 = (1,2,3,5,6,7,8,9)
	im5 = (1,2,3,4,6,7,8,9)
	im6 = (1,2,3,4,5,7,8,9)
	im7 = (1,2,3,4,5,6,8,9)
	im8 = (1,2,3,4,5,6,7,9)
	im9 = (1,2,3,4,5,6,7,8)
	return((im1\im2\im3\im4\im5\im6\im7\im8\im9))
}

void matrixsetup(real matrix Sig, real matrix coefS, real matrix condsdS){
	S1 = Sig
	temp = (Sig[2,.]\Sig[1,.]\Sig[3..9,.])
	S2 = (temp[.,2],temp[.,1],temp[.,3..9])
	temp = (Sig[3,.]\Sig[1..2,.]\Sig[4..9,.])
	S3 = (temp[.,3],temp[.,1..2],temp[.,4..9])
	temp = (Sig[4,.]\Sig[1..3,.]\Sig[5..9,.])
	S4 = (temp[.,4],temp[.,1..3],temp[.,5..9])
	temp = (Sig[5,.]\Sig[1..4,.]\Sig[6..9,.])
	S5 = (temp[.,5],temp[.,1..4],temp[.,6..9])
	temp = (Sig[6,.]\Sig[1..5,.]\Sig[7..9,.])
	S6 = (temp[.,6],temp[.,1..5],temp[.,7..9])
	temp = (Sig[7,.]\Sig[1..6,.]\Sig[8..9,.])
	S7 = (temp[.,7],temp[.,1..6],temp[.,8..9])
	temp = (Sig[8,.]\Sig[1..7,.]\Sig[9,.])
	S8 = (temp[.,8],temp[.,1..7],temp[.,9])
	temp = (Sig[9,.]\Sig[1..8,.])
	S9 = (temp[.,9],temp[.,1..8])


	coefS1 = S1[1,2..9]*cholinv(S1[2..9,2..9])
	condS1 = S1[1,1] - coefS1*S1[2..9,1]
	coefS2 = S2[1,2..9]*cholinv(S2[2..9,2..9])
	condS2 = S2[1,1] - coefS2*S2[2..9,1]
	coefS3 = S3[1,2..9]*cholinv(S3[2..9,2..9])
	condS3 = S3[1,1] - coefS3*S3[2..9,1]
	coefS4 = S4[1,2..9]*cholinv(S4[2..9,2..9])
	condS4 = S4[1,1] - coefS4*S4[2..9,1]
	coefS5 = S5[1,2..9]*cholinv(S5[2..9,2..9])
	condS5 = S5[1,1] - coefS5*S5[2..9,1]
	coefS6 = S6[1,2..9]*cholinv(S6[2..9,2..9])
	condS6 = S6[1,1] - coefS6*S6[2..9,1]
	coefS7 = S7[1,2..9]*cholinv(S7[2..9,2..9])
	condS7 = S7[1,1] - coefS7*S7[2..9,1]
	coefS8 = S8[1,2..9]*cholinv(S8[2..9,2..9])
	condS8 = S8[1,1] - coefS8*S8[2..9,1]
	coefS9 = S9[1,2..9]*cholinv(S9[2..9,2..9])
	condS9 = S9[1,1] - coefS9*S9[2..9,1]

	coefS = (coefS1\coefS2\coefS3\coefS4\coefS5\coefS6\coefS7\coefS8\coefS9)
	condvarS = (condS1\condS2\condS3\condS4\condS5\condS6\condS7\condS8\condS9)
	condsdS = condvarS:^.5
}


void CheckAddingUp(real matrix Util, real matrix cmUtil, real matrix ee, ///
real matrix lnPP, real matrix cmlnPP, real matrix uu){
	if(max(abs(Util - cmUtil - ee)) > 1e-5) ///
		exit(_error("Util adding up problem."))  
	if(max(abs(lnPP - cmlnPP - uu)) > 1e-5) ///
		exit(_error("lnPP adding up problem."))
}


/* These functions draw from a normal with mean cm and standard deviation sig, truncated 
above at ub, or below at lb. Should eventually use Geweke's faster algorithm. Note that 
invnormal is always evaluated at an argument between 0 and 0.5. This is for numerical 
purposes - invnormal is more accurate close to 0 than close to 1 (its domain is 1e-323 to 
1-1e-16). Also runiform(1,1) can return 0, hence the use of the while loop. */

real scalar utrnorm(real scalar ub, real scalar cm, real scalar sig){
	nrm = 0
	while(nrm==0){
		nrm = runiform(1,1)*normal((ub - cm)/sig)
	}	
	if(nrm<.5) return(cm + sig*invnormal(nrm))
	else return(cm - sig*invnormal(1-nrm))
}


real scalar ltrnorm(real scalar lb, real scalar cm, real scalar sig){
	return(-utrnorm(-lb,-cm,sig))
}

/* Test the truncated normal draws. */
/* 
a = J(100000,1,.)
b = J(100000,1,.)
sig = 3
cm = -0.3
lb = 1.9
ub = -.07
for(ii=1;ii<=100000;ii++){
	a[ii] = utrnorm(ub,cm,sig)
	b[ii] = ltrnorm(lb,cm,sig)
}
alpha = (lb - cm)/sig
beta = (ub - cm)/sig

mean(a)
actualmeana = cm - sig*(normalden(beta)/normal(beta))
(mean(a) - actualmeana)/actualmeana

variance(a)
actualvara = sig^2*(1 - (beta*normalden(beta))/normal(beta) - /// 
	(normalden(beta)/normal(beta))^2)
(variance(a) - actualvara)/actualvara

max(a)

mean(b)
actualmeanb = cm + sig*(normalden(alpha)/(1-normal(alpha)))
(mean(b) - actualmeanb)/actualmeanb

variance(b)
actualvarb = sig^2*(1 + (alpha*normalden(alpha))/(1-normal(alpha)) - /// 
	(normalden(alpha)/(1-normal(alpha)))^2)
(variance(b) - actualvarb)/actualvarb
min(b)
 */

/* This returns the conditional mean of one error (the one with the index "current") given 
the other errors. */
real matrix cm(real scalar current, real matrix ue, real matrix coefS, ///
real matrix indmat){
		index = indmat[current,.]
		return(ue[.,index]*coefS[current,.]')
}

/* This checks that no errors are missing, and that the error vector implies the observed 
choice. */
void checkchoice(real matrix cmUtil_ii, real matrix ue_ii, real scalar Proc_ii){
	if(hasmissing(ue_ii)) exit(_error("Missing error."))						
	ImpliedChoice = 0
	temp = J(1,2,0)
	Util_ii = cmUtil_ii + ue_ii[.,6..9]	
	maxindex((0,Util_ii),1,ImpliedChoice,temp) 	/* Include 0, the utility from procedure 
		1. */
	if(Proc_ii != ImpliedChoice) exit(_error("Inconsistent with observed choice."))				
}

/* This returns the part of the conditional mean of utility associated with payments. */

real matrix cmUtillnPX(real scalar TypebP, real matrix bP, real matrix cmlnPP, ///
real matrix NonResp, real matrix RegNum, real matrix Year, real matrix HHI_Thirds){
	cmlnPX = SubtractFirstCol(cmlnPP)
	if (TypebP == 0) return(cmlnPX*bP)
	if (TypebP == 1) return(cmlnPX:*((1 :- NonResp)*bP[1] + NonResp*bP[2]))
	if (TypebP == 2) return(cmlnPX:*((RegNum :== 1)*bP[1] + (RegNum :== 2)*bP[2] + /// 
		(RegNum :== 3)*bP[3] + (RegNum :== 4)*bP[4]))
	if (TypebP == 3){
		outmat = J(rows(cmlnPX),cols(cmlnPX),0)
		for(ii = 2002; ii <= 2007; ii++){  /* Hardcoding year range here. */
			outmat = outmat + cmlnPX:*(Year :== ii)*bP[ii - 2002 + 1]
		}
		return(outmat)
	} 
	/* bP can be 1x5 or 5x1 here. */
	if (TypebP == 4) return(SubtractFirstCol(cmlnPP:*(bP[1], bP[2], bP[3], bP[4], bP[5]))) 
	if (TypebP == 5) return(SubtractFirstCol(cmlnPP:*(bP[1], bP[2], bP[2], bP[2], bP[2])))
 	if (TypebP == 6) return(cmlnPX:*((HHI_Thirds :== 1)*bP[1] + ///
									 (HHI_Thirds :== 2)*bP[2] + ///
									 (HHI_Thirds :== 3)*bP[3]))
}	

/* This finds the chosen procedure (Choice), the payment for the chosen procedure (Paymt), 
and the patient utility (UtilPatient), from one set of utility and price draws.  */
void SimOutcomesOnce(real matrix cmUtilPatient, real matrix cmUtil, ///
real matrix cmlnPP, real matrix Sig, real matrix Choice, real matrix Paymt, ///
real matrix UtilPatient, real scalar FixedSeed, | real scalar seed){
	if (FixedSeed) rseed(seed)
	UtilErrs = (cholesky(Sig[6..9, 6..9])*rnormal(4, rows(cmUtil), 0, 1))'
	PhyUtil = cmUtil + UtilErrs /* These utilities determine treatment choice. */
	Choice = J(rows(cmUtil), 1, 0)		
	Paymt = J(rows(cmUtil), 1, 0)
	UtilPatient = J(rows(cmUtil), 1, 0)	
	i = 0
	w = 0
	cholinvSig = cholinv(Sig[6..9, 6..9])
	for (ii=1; ii <= rows(cmUtil); ii++){
		maxindex((0, PhyUtil[ii, .]), 1, i, w)
		Choiceii = min(i)
		Choice[ii] = Choiceii
		 /* UtilPatient is the non-financial part of utility. Note that need to subtract 1 
		from the indices when the utility-maximizing choice is not 1. */
		if (Choiceii == 1) UtilPatient[ii] = 0
		else UtilPatient[ii] = cmUtilPatient[ii, Choiceii - 1] + ///
			UtilErrs[ii, Choiceii - 1]
		/* Draw from payment distribution for chosen treatment, conditioning on the 
		utility error draw. */
		PayErrorMean = Sig[Choiceii, 6..9]*cholinvSig*UtilErrs[ii, .]'
		PayErrorVar = Sig[Choiceii, Choiceii] - /// 
			Sig[Choiceii, 6..9]*cholinvSig*Sig[6..9, Choiceii]
		PayError = PayErrorMean + rnormal(1, 1, 0, PayErrorVar^.5)
		Paymt[ii] = exp(cmlnPP[ii, Choiceii] + PayError)
	}
}

/* Return the share of each choice, given the procedure choice vector. */
real matrix Fracs(real matrix Choice){
	assert(cols(Choice) == 1)
	return(rows(Choice)^-1*(sum(Choice :== 1), sum(Choice :== 2), sum(Choice :== 3), ///
		sum(Choice :== 4), sum(Choice :== 5)))
}

/* Returns the means of X for each value of the integer-valued variable Y. */
real matrix MeansByX(real matrix X, real matrix Y){
	Out = J(min(X), max(X), 0)
	for (ii = min(X); ii <= max(X); ii++){
		Out[ii] = sum((X :== ii):*Y)/sum(X :== ii)
	}
	return(Out)
}

/* This mimics the stata command bys region: tab Procedure. */
real matrix BysRegTabProc(real matrix R, real matrix P){
	assert(min(R) == 1 & max(R) == 4)
	assert(min(P) == 1 & max(P) == 5)
	Out = J(4, 5, .)
	for (r = 1; r <= 4; r++){
		for (p = 1; p <= 5; p++){
			Out[r, p] = sum((R :== r):*(P :== p))/sum(R :== r)
		}
	}
	return(Out)
}

/* This finds outcomes (procedures and prices). It calls SimOutcomesOnce many times and 
averages the procedure and price results. */
void FO(real matrix cmUtil, real matrix cmlnPP, real matrix Sig, real matrix UL, ///
	real matrix Choice, real matrix Paymt){
	Reps =  25  /* Number of draws to average over. */
	ChoiceMat = J(Reps, 5, 0) /* 5 for number of procedures. */
	PaymtMat = ChoiceMat
	for (ii = 1; ii <= Reps; ii++){
		SimOutcomesOnce(cmUtil*., cmUtil, cmlnPP, Sig, Choiceii = ., ///
			Paymtii = ., UtilPatientii = ., FixedSeed = 0) /* Note passing missing values 
			in for patient utility (the first argument). This function doesn't calculate
			utilities. */
		TruncPaymtii = rowmax((Paymtii, J(rows(Paymtii), 1, UL[2])))
		TruncPaymtii = rowmin((TruncPaymtii, J(rows(Paymtii), 1, UL[1])))
		ChoiceMat[ii, .] = Fracs(Choiceii)
		PaymtMat[ii, .] = MeansByX(Choiceii, TruncPaymtii)	
	}
	Choice = mean(ChoiceMat)
	Paymt = mean(PaymtMat)
}

/* Simulates patient's welfare loss between the outcomes with financial incentives and
without. The non-financial incentives part of the conditional mean of utility is made up
of two parts: it is patient utility minus the physician's cost of providing the treatment.
This second part is a constant fraction of the PPO payment (by assumption service
quantities don't vary by plan type, so costs shouldn't, so fixing the plan type). If wgh =
0, costs are constant and changes in cmUtilNonFin equal changes in patient utility.
Bundled payments are then optimal for the patient. If wgh = 1, then the optimal outcome
for the patient would be achieved if everyone were in PPOs. */

void WL(real matrix cmUtilNonFin, real matrix cmUtil, real matrix cmlnPP, ///
real matrix PPO_cmUtilFin, real matrix Sig, real scalar bP, real scalar wgh, ///
real scalar fd, real scalar wl, real scalar wlf){
nobs = rows(cmlnPP)
Reps = 25 /* Number of draws to average over. */
fdvec = J(nobs, 1, .)
wlvec = J(nobs, 1, .)
wlfvec = J(nobs, 1, .)
for (ii = 1; ii <= Reps; ii++){
	seed = floor(1e4*runiform(1,1))
	cmUtilPatient = cmUtilNonFin + wgh*PPO_cmUtilFin 
	SimOutcomesOnce(cmUtilPatient, cmUtil, cmlnPP, Sig, Proc = ., ///
		PaymtFin = ., Util1 = ., FixedSeed = 1, seed) /* This is what happens under the 
		current system of FFS payments, with choices determined by cmUtil. Important to 
		use the same seed in both calculations. */
	SimOutcomesOnce(cmUtilPatient, cmUtilNonFin, cmlnPP, Sig, ProcNonFin = ., ///
		PaymtNoFin = ., Util2 = ., FixedSeed = 1, seed) /* This is what happens under the 
		bundled payments counterfactual, with choices determined by cmUtilNonFin. */

	if (wgh == 0) assert(min(Util2 :>= Util1) == 1) /* The non-financial part of utility 
		should be maximized when it is what determines the choice. */	
	cmlnPPProcNonFin = Extr(cmlnPP, ProcNonFin)
	cmlnPPProc = Extr(cmlnPP, Proc)
 	DollarUtilDiff = (exp(cmlnPPProcNonFin):* ///
		(exp((Util2 - Util1)/bP) :- 1))*1000 

	UtilDiffFrac = (DollarUtilDiff/1000):/exp(cmlnPPProc)
	fdvec[ii] = sum(DollarUtilDiff :!= 0)/nobs /* Frac with diff procs. */
	wlvec[ii] = mean(DollarUtilDiff) /* Welfare loss. */
	wlfvec[ii] = mean(UtilDiffFrac)	/* Welfare loss as a fraction of payment. */
}
fd = mean(fdvec)
wl = mean(wlvec)
wlf = mean(wlfvec)	
}

/* This takes in a n x 5 matrix and a n x 1 vector. The vector contains the column indices 
of the returned elements. */
real Extr(real matrix A, real matrix I){
	Out = J(rows(A), 1, .)
	for (ii = 1; ii <= rows(A); ii++) Out[ii] = A[ii, I[ii]]	
	return(Out)
}


/* This returns the fraction of people receiving each treatment, given the predicted 
utilities and the utility error variance. This fraction is random but the variance will be 
small since the sample size is relatively large. WelfareAndPrices calculates more
variables of interest than just the procedure shares. */
real matrix FracProc(real matrix pu, real matrix Sig){
	UtilErrs = (cholesky(Sig[6..9, 6..9])*rnormal(4, rows(pu), 0, 1))'
	puerr = pu + UtilErrs
	temp = J(rows(pu), 1, 0)
	i = 0
	w = 0
	for(ii = 1; ii <= rows(pu); ii++){
		maxindex((0, puerr[ii,.]), 1, i, w)
		temp[ii] = min(i)
	}
	return(Fracs(temp))
}


/* This computes the number of observations which have the correctly predicted choice of 
procedure. */
real matrix NumPred(real matrix PredUtil, real matrix Procedure){
	nobs = rows(Procedure)
	Prediction = J(nobs,1,0)
	i = 0
    w = 0
    for(ii=1;ii<=nobs;ii++){
    	maxindex((0,PredUtil[ii,.]),1,i,w)
    	Prediction[ii] = min(i)
    }
    return(sum(Procedure:==Prediction))
}


/* This computes the mean of the draws excluding the first BurnIn draws. BurnIn
can be a fraction (e.g Draws/2 for odd Draws.) */
real matrix MeanAndSDWithBurnIn(real matrix Draws, real scalar BurnIn, /// 
real matrix Est, real matrix SD, real matrix Out){
	Est = mean(Draws[ceil(BurnIn)..rows(Draws),.])
	SD = (diagonal(variance(Draws[ceil(BurnIn)..rows(Draws),.]))'):^.5
	Out = (Est\SD)
}


real matrix SubtractFirstCol(real matrix pmat){
	assert(cols(pmat) == 5)
	return((pmat[.,2]-pmat[.,1],pmat[.,3]-pmat[.,1],pmat[.,4]-pmat[.,1], ///
		pmat[.,5]-pmat[.,1]))
}


/* Draw the utility errors given the observed choice by gibbs sampling. */
real matrix DrawUtilErrors(real scalar hh, real matrix ue_ii, ///
	real scalar Procedure_ii, real scalar lnPrice_ii, real matrix FE_ii, ///
	real matrix aFE_Procedure_ii, real matrix WWA_ii, real matrix aW_Procedure_ii, ///
	real scalar InsaIns_ii_Procedure_ii, real matrix cmUtil_ii, real matrix coefS, ///
	real matrix indmat, real matrix condsdS){
		 /* ue_ii[Procedure_ii] is the observed price error. */
		ue_ii[Procedure_ii] = lnPrice_ii - FE_ii*aFE_Procedure_ii -  ///
			WWA_ii*aW_Procedure_ii - InsaIns_ii_Procedure_ii
		cmUtilDot_ii = (.,cmUtil_ii) /* Add missing value at beginning so that the jth 
		entry corresponds to the jth procedure. */
			
		if(Procedure_ii != 1){
			/* Draw utility error for chosen procedure. */
			current = 4 + Procedure_ii /* Offset of 4 so that utility error for Procedure 
				== 2 is in the 6th spot, right after the price errors. */		
			lbs = .
			for(jj = 2; jj <= 5; jj++){
				if(jj == Procedure_ii) continue
				lbs = (lbs,ue_ii[4 + jj] - cmUtilDot_ii[Procedure_ii] + cmUtilDot_ii[jj])
			}
			lb = max((lbs,-cmUtilDot_ii[Procedure_ii]))
			cm = cm(current,ue_ii,coefS,indmat)		
			ue_ii[current] = ltrnorm(lb,cm,condsdS[current])
			checkchoice(cmUtil_ii,ue_ii,Procedure_ii)
	
			/* Draw utility error for other procedures (no utility error for procedure == 
			1, since errors already differenced. */
			for(jj = 2; jj <= 5; jj++){
				if(jj == Procedure_ii) continue /* Already drew the utility error for the 
				chosen procedure. */
				current = 4 + jj		
				ub = cmUtilDot_ii[Procedure_ii] - cmUtilDot_ii[jj] + ///
					ue_ii[4 + Procedure_ii]
				cm = cm(current,ue_ii,coefS,indmat)
				ue_ii[current] = utrnorm(ub,cm,condsdS[current])
				checkchoice(cmUtil_ii,ue_ii,Procedure_ii)
			}
	
			/* Draw price errors for unchosen procedures. */
			for(jj = 1; jj <= 5; jj++){
				if(jj == Procedure_ii) continue	/* Skip - price error is observed for 
					chosen procedure. */
				cm = cm(jj,ue_ii,coefS,indmat)			
				ue_ii[jj] = rnormal(1,1,cm,condsdS[jj])
				checkchoice(cmUtil_ii,ue_ii,Procedure_ii)
			}
		}
	
		if(Procedure_ii == 1){
			/* Draw utility errors for unchosen procedures (again, no utility error for 
			procedure == 1). */
			for(jj = 2; jj <= 5; jj++){
				current = 4 + jj
				ub = -cmUtilDot_ii[jj]
				cm = cm(current,ue_ii,coefS,indmat)		
				ue_ii[current] = utrnorm(ub,cm,condsdS[current])
				if(hh>1) checkchoice(cmUtil_ii,ue_ii,Procedure_ii)
				/* The very first time through the GS, errors might not be consistent with 
					the observed choice. But once they are, they will always be. */
	
			}	
	
			/* Draw price errors for unchosen procedures. */
			for(jj = 2; jj <= 5; jj++){
				cm = cm(jj,ue_ii,coefS,indmat)				
				ue_ii[jj] = rnormal(1,1,cm,condsdS[jj])
				checkchoice(cmUtil_ii,ue_ii,Procedure_ii)
			}
		}
	return(ue_ii)
	}
	

/* Declare the parameter estimates structure. Will save final output in such a structure, 
using SaveParamEstStruct(). */
struct ParamEstStruct{
	real matrix aFE, aW, aIns, bFE, bW, bP, Sig
	real scalar TypebP	
}

void SaveParamEstStruct(real matrix aFE, real matrix aW, real matrix aIns, ///
real matrix bFE, real matrix bW, real matrix bP, real matrix Sig, real scalar TypebP, ///
string scalar CodeDir, string scalar Name){
	struct ParamEstStruct scalar Est
		Est.aFE    = aFE 
		Est.aW     = aW		
		Est.aIns   = aIns 
		Est.bFE    = bFE 
		Est.bW     = bW		
		Est.bP     = bP	
		Est.Sig    = Sig 
		Est.TypebP = TypebP
	Out = CodeDir + "/Est" + Name
	fh = fopen(Out, "rw")
    fputmatrix(fh, Est)
    fclose(fh)
}

/* Use the structure Est to set the starting values of the parameter matrices. Not setting 
aIns here, since it comes from the service data. */
void GetEst(real matrix aFE, real matrix aW, real matrix bFE, real matrix bW, ///
real matrix bP, real matrix Sig, real scalar TypebP, string scalar CodeDir, ///
string scalar Name){	
		Out = CodeDir + "/Est" + Name
		fh = fopen(Out,"rw")
		Est =  fgetmatrix(fh)
		fclose(fh)
		SubGetEst(aFE, aW, bFE, bW, bP, Sig, TypebP, Est)
}

void SubGetEst(real matrix aFE, real matrix aW, real matrix bFE, real matrix bW, ///
real matrix bP, real matrix Sig, real scalar TypebP, struct ParamEstStruct scalar Est){
		aFE    = Est.aFE 
		aW     = Est.aW		 
		bFE    = Est.bFE 
		bW     = Est.bW		
		bP     = Est.bP	
		Sig    = Est.Sig 
		TypebP = Est.TypebP
}

/* Declare the parameter draws structure. Will save final output in such a structure, 
using SaveParamDrawsStruct(). */
struct ParamDrawsStruct{
	real matrix DrawsaFE, DrawsaW, DrawsbFE, DrawsbW, DrawsbP, DrawsSig
}

void SaveParamDrawsStruct(real matrix DrawsaFE, real matrix DrawsaW, ///
real matrix DrawsbFE, real matrix DrawsbW, real matrix DrawsbP, real matrix DrawsSig, /// 
string scalar CodeDir, string scalar Name){
	struct ParamDrawsStruct scalar Draws
		Draws.DrawsaFE = DrawsaFE 
		Draws.DrawsaW  = DrawsaW		
		Draws.DrawsbFE = DrawsbFE 
		Draws.DrawsbW  = DrawsbW		
		Draws.DrawsbP  = DrawsbP	
		Draws.DrawsSig = DrawsSig 
	Out = CodeDir + "/Draws" + Name
	fh = fopen(Out,"rw")
    fputmatrix(fh,Draws)
    fclose(fh)
}

/* Use the structure Draws to set the starting values of the parameter matrices. Not 
setting DrawsaIns here, since aIns comes from the service data. */
void GetDraws(real matrix DrawsaFE, real matrix DrawsaW, real matrix DrawsbFE, ///
real matrix DrawsbW, real matrix DrawsbP, real matrix DrawsSig, string scalar CodeDir, ///
string scalar Name){	
		Out = CodeDir + "/Draws" + Name
		fh = fopen(Out,"rw")
		Draws =  fgetmatrix(fh)
		fclose(fh)
		SubGetDraws(DrawsaFE, DrawsaW, DrawsbFE, DrawsbW, DrawsbP, DrawsSig, Draws)
}

void SubGetDraws(real matrix DrawsaFE, real matrix DrawsaW, real matrix DrawsbFE, ///
real matrix DrawsbW, real matrix DrawsbP, real matrix DrawsSig, ///
struct ParamDrawsStruct scalar Draws){
		DrawsaFE = Draws.DrawsaFE 
		DrawsaW  = Draws.DrawsaW		 
		DrawsbFE = Draws.DrawsbFE 
		DrawsbW  = Draws.DrawsbW		
		DrawsbP  = Draws.DrawsbP	
		DrawsSig = Draws.DrawsSig 
}

/* Structures of this type are created by StandardErrors and used when bootstrapping 
the average partial effects' standard errors. */
struct SEDrawsStruct{
	real matrix DaFE, DaW, DaIns, DbFE, DbW, DbP, DSig
}

void SaveSEDrawsStruct(real matrix DaFE, real matrix DaW, real matrix DaIns, ///
real matrix DbFE, real matrix DbW, real matrix DbP, real matrix DSig, ///
string scalar CodeDir, string scalar Name){
	struct SEDrawsStruct scalar D
		D.DaFE  = DaFE 
	    D.DaW   = DaW  
       	D.DaIns = DaIns
        D.DbFE  = DbFE 
        D.DbW   = DbW  
        D.DbP   = DbP  
        D.DSig  = DSig
	Out = CodeDir + "/SEDraws" + Name
	fh = fopen(Out, "rw")
	fputmatrix(fh, D)
	fclose(fh)
}              


real matrix DisplayaIns(real matrix aIns){
	Output = aIns*0
	/* In each region, plan types are ordered as Comprehensive, HMO, POS. Order as HMO, 
	POS, Comprehensive. */
	for(ii = 1; ii <= 4; ii++){
		Output[3*(ii-1) + 1..3*ii, .] = SwitchPT(aIns[3*(ii-1) + 1..3*ii, .])
	}
	return(Output*100)
}


real matrix SwitchPT(real matrix A){
	return((A[2..3,.]\A[1,.]))
}

/* Average each hundred rows of some input matrix. Useful when analysing output. */
real matrix AvgH(real matrix A){
	dim = rows(A)/100
	B = J(dim,cols(A),.)
	assert(round(dim) == dim)
	for(ii=1;ii<=dim;ii++){
		B[ii,.] = mean(A[1 + (ii-1)*100..ii*100,.])
	}
	return(B)
}

/* Draw from the posterior of the variance matrix distribution, given the restrictions
imposed (unrestricted, block diagonal, or diagonal). */
real matrix DrawSig(real matrix PriorDoF, real scalar nobs, real matrix invPriorScale, ///
real scalar PriorGammaAlpha, real scalar PriorGammaBeta, real matrix uu, ///
real matrix ee, real scalar SigType){
	/* Unrestricted covariance. */
	if (SigType == 0){
		errs = (uu, ee)
		Sig = DrawWishart(PriorDoF, nobs, invPriorScale, errs)
	}	
	/* Block diagonal covariance (independent price and utility errors). */	
	if (SigType == 1){
		Sig_uu = DrawWishart(PriorDoF, nobs, invPriorScale[|1, 1 \ 5, 5|], uu)
		Sig_ee = DrawWishart(PriorDoF, nobs, invPriorScale[|6, 6 \ 9, 9|], ee)		
		Sig = (Sig_uu, J(5, 4, 0) \ J(4, 5, 0), Sig_ee)
	}
	/* Diagonal covariance. */
	if (SigType == 2){
		Sig = J(9,9,0)
		ue = (uu, ee)
		for (ii = 1; ii <= cols(ue); ii++){
			Sig[ii, ii] = rgamma(1, 1, PriorGammaAlpha + nobs/2, (PriorGammaBeta^-1 + /// 
				cross(ue[., ii], ue[., ii])/2)^-1)^-1 	
		}
		assert(isdiagonal(Sig))
	}
	return(Sig)
}

real matrix DrawWishart(real matrix PriorDoF, real scalar nobs, ///
real matrix invPriorScale, real matrix errs){
	PostDoF = PriorDoF + nobs
	PostScale = luinv(invPriorScale + errs'*errs)
	dim = cols(errs)
	TT = J(dim, dim, 0)
	for (ii = 1; ii <= dim; ii++){
		for (jj = 1; jj <= ii; jj++){	
			if (ii == jj) TT[ii, jj] = rchi2(1, 1, PostDoF - ii + 1)^0.5
			else if (ii > jj) TT[ii, jj] = rnormal(1, 1, 0, 1)
		}
	}
	chPS = cholesky(PostScale)
	invSig = chPS*TT*TT'*chPS'
	Sig = luinv(invSig)
	return(Sig)
}

/* Form the aIns matrix, using the region-specific service weights. */
real matrix GetRelevantaInsData(real matrix A){
	assert(rows(A) == 20)
	assert(cols(A) == 12)
	aIns = (A[1, 1..3], A[2, 4..6], A[3, 7..9], A[4, 10..12] \ ///
			A[5, 1..3], A[6, 4..6], A[7, 7..9], A[8, 10..12] \ ///
			A[9, 1..3], A[10, 4..6], A[11, 7..9], A[12, 10..12] \ ///
			A[13, 1..3], A[14, 4..6], A[15, 7..9], A[16, 10..12] \ ///
			A[17, 1..3], A[18, 4..6], A[19, 7..9], A[20, 10..12])
	return(aIns)		
}

/* This returns what the Ins matrix would be if everyone were in their region's 
Comprehensive plan if pt = 2, HMO if pt = 4... */
real matrix MakeCfactPT(real scalar nobs, real scalar pt, real matrix RegNum){
	Ins = J(nobs, 12, 0)
	if (pt != 4){ /* PPO are the omitted category. Corresponds all zeros in Ins. */	
		ind = 3*(RegNum :- 1) :+ pt /* Different regions & same plan types -> 
			different columns filled with ones. */
			for (jj = 1; jj <= nobs; jj++) Ins[jj, ind[jj]] = 1 
	}
	return(Ins)
}

/* Turn a row (or column) vector like vec(aFE)' back into aFE. */
real matrix ToMatA(real matrix A){
	return(rowshape(A, 5)')
}

/* Turn a row (or column) vector like vec(bFE)' back into bFE. */
real matrix ToMatB(real matrix B){
	return(rowshape(B, 4)')
}

/* Take the SaveSEDrawsStruct and an index, and return the parameter matrices 
corresponding to that index. */
void GetSEDrawsii(struct SEDrawsStruct scalar D, real scalar ii, aFEii, aWii, aInsii, ///
bFEii, bWii, bPii, Sigii){
	aFEii  = ToMatA(D.DaFE[ii, .])
	aWii   = ToMatA(D.DaW[ii, .])
	aInsii = ToMatA(D.DaIns[ii, .])
	bFEii  = ToMatB(D.DbFE[ii, .])
	bWii   = ToMatB(D.DbW[ii, .])
	bPii   = D.DbP[ii, .]
	Sigii  = invvech(D.DSig[ii,.]')
}


/* Test equality of bP's by procedure */
real scalar TestEqbP(real matrix AllbP, real scalar BurnIn){
	bP = mean(AllbP[ceil(BurnIn)..rows(AllbP),.])'
	Var = variance(AllbP[ceil(BurnIn)..rows(AllbP),.])
	R = (1, -1, 0, 0, 0 \ ///
		 1, 0, -1, 0, 0 \ ///
		 1, 0, 0, -1, 0 \ ///
		 1, 0, 0, 0, -1)
	stat = bP'*R'*luinv(R*Var*R')*R*bP
	return(chi2(5,stat))
}
end

