/* Run some regressions to see how plan type predicts the probability of various 
conditions, e.g. AMI, other forms of ischemic heart disease, stroke, conditional on other 
covariates. */

capture log close
clear all
macro drop _all
set more off
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
global Conditions AMI AcuteIschHD AnginaPec ChronicIschHD Cancer COPD CereVas Diabetes /// 
	FluPneu KidneyDs
do ${Code}/DefineGlobals.do /* This is for the CtyVar macros, used below. */


/* For each year convert monthly enrollment data into a dataset with one observation per 
patient per year. Need to do this because the available yearly enrollment datasets 
aren't for the full range of years required. */
capture log close
log using ${Code}/Logs/CheckSelection, replace
if (1){
forvalues ii = 2002/2007{
	use ${Data}/`ii'/ccaet_all_`ii', clear /* These files are converted from the source 
		datasets by ConvertEnrollment.stc. */
	sort enrolid dtstart, stable
	by enrolid: egen SumMemDays = sum(memdays)
	keep if inlist(SumMemDays,365,366)
	by enrolid: keep if _n == 1
	compress
	capture drop phyflag rx wgtkey plankey mhsacovg dtstart dtend /* Drop unused 
		variables. */
	save ${Data}/`ii'/ccaet`ii'_annual, replace
}

/* Create a dataset of inpatient admissions for conditions of interest for all years. */
forvalues ii = 2002/2007{
	use ${Data}/`ii'/ccaei`ii', clear
	gen pdx_first3 = substr(pdx,1,3)
	destring pdx_first3, generate(numpdx_first3) force
	gen pdx_first4 = substr(pdx,1,4)
	destring pdx_first4, generate(numpdx_first4) force
	sort enrolid
	by enrolid: egen AMI = max(pdx_first3 == "410")
	by enrolid: egen AcuteIschHD = max(pdx_first3 == "411") /* "Other acute and subacute 
		forms of ischemic heart disease" */
	by enrolid: egen AnginaPec = max(pdx_first3 == "413") 
	by enrolid: egen ChronicIschHD = max(pdx_first3 == "414") /* "Other forms of chronic 
		ischemic heart disease" */
	by enrolid: egen Cancer = max(inrange(numpdx_first3,140,208)) /* Malignant neoplasms. 
		*/
	by enrolid: egen COPD = max(inrange(numpdx_first3,490,496)) /* COPD and allied 
		conditions. */
	by enrolid: egen CereVas = max(inrange(numpdx_first3,430,438))
	by enrolid: egen Diabetes = max(pdx_first3 == "250")
	by enrolid: egen FluPneu = max(inrange(numpdx_first3,480,487))
	by enrolid: egen KidneyDs = max(inrange(numpdx_first3,580,589)) /* Nephritis, 
		nephrotic syndrome and nephrosis. */
	by enrolid: egen modalempcty = mode(empcty), minmode	/* For those with multiple 
		distinct empcty modes recorded. */
	replace empcty = modalempcty	
	keep if AMI | AcuteIschHD | AnginaPec | ChronicIschHD | Cancer | COPD | CereVas | ///
		Diabetes | FluPneu | KidneyDs
	keep enrolid year empcty ${Conditions}
	duplicates drop
	compress
	save ${Data}/`ii'/ccaei`ii'_checkselection, replace
}


forvalues ii = 2002/2007{
	use ${Data}/`ii'/ccaet`ii'_annual, clear
	gen year = `ii'	
	merge 1:1 enrolid year using ${Data}/`ii'/ccaei`ii'_checkselection, keep(master match) 
	recode age (0/39 = 1) (40/44 = 2) (45/49 = 3) (50/54 = 4) (55/59 = 5) ///
		(60/max = 6), gen(agegroup)
	destring egeoloc eeclass eestatu indstry sex, replace
	replace indstry = 0 if mi(indstry)
	foreach jj in ${Conditions}{
		replace `jj' = 0 if mi(`jj')
		replace `jj' = 100*`jj'
	}
	table plantyp, c(m AMI n AMI) row	
	save ${Data}/`ii'/CheckSelectionRegs_`ii', replace	
}

use ${Data}/2002/CheckSelectionRegs_2002, clear
forvalues ii = 2003/2007{
	append using ${Data}/`ii'/CheckSelectionRegs_`ii'
}

capture drop _merge
merge m:1 empcty using ${Data}/ARF_Data/arfclean2011.dta, keepusing(${CtyVars}) /// 
	keep(match)
bys egeoloc: egen temp = median(FracMed)
replace FracMed = temp if mi(FracMed) /* Fill in the few observations with missing 
	FracMed with state medians. */
drop temp
compress
save ${Data}/CheckSelection, replace

table plantyp, c(m AMI n AMI) row

misstable sum ${Conditions} agegroup year egeoloc sex urban ${CtyVars} plantyp enrolid /// 
	if inlist(plantyp,2,4,5,6)

/* The regressors here are those in the main model, but omitting the variables which are 
only observed for inpatients (e.g. the clinical and physician covariates).*/
foreach jj in $Conditions{
	eststo reg`jj':	reg `jj' i.agegroup i.year i.egeoloc i.sex i.urban  /// 
		${CtyVars} b6.plantyp if inlist(plantyp,2,4,5,6), cluster(enrolid) 
}
esttab regAMI regAcuteIschHD regAnginaPec regChronicIschHD regCancer regCOPD ///
	regCereVas regDiabetes regFluPneu regKidneyDs, keep(2.plantyp 4.plantyp 5.plantyp) /// 
	noobs se b(%9.4f)

sum ${Conditions} if e(sample)
/* reg AMI i.year i.age i.egeoloc i.urban i.eeclass i.eestatu i.indstry i.sex i.plantyp if 
inlist(plantyp,2,4,5,6)
reg AMI i.year i.agegrp i.egeoloc i.msa i.eeclass i.eestatu i.indstry i.sex i.plantyp if 
inlist(plantyp,2,4,5,6)
reg AMI i.year i.age i.egeoloc i.msa i.eeclass i.eestatu i.indstry i.sex i.plantyp if 
inlist(plantyp,2,4,5,6) */

log close
