/* This file looks at other conditions (prostate cancer, breast cancer, inguinal hernia
and spinal disc hernia) to see if there is a relation between payments as they vary across
plan types and treatments. */

clear
global Data /disk/agedisk2/medstat.work/dcoey/Data


/* -------------------------------------------------------------------------------------*/
/* Part A. Convert the outpatient data into a dataset in which there is one observation */ 
/* per enrolid X condition, with indicators for "intensive" treatment of each of the    */
/* four conditions and a variable recording the total amount spent on that treatment.   */
/* -------------------------------------------------------------------------------------*/

use ${Data}/2002/ccaeo2002_othrconds /* The othrconds files are made by 
	ConvertToStata.stc.  */
forvalues ii = 2003/2007{
	append using ${Data}/`ii'/ccaeo`ii'_othrconds
}
compress
rename DX* dx*
rename PROC1 proc1

keep enrolid year seqnum pay plantyp dx1 dx2 proc1 proctyp
gen byte PrstCncr = 0
gen byte BrstCncr = 0
gen byte IngHrn = 0
gen byte SpDscHrn = 0
qui forvalues ii = 1/2{
	replace PrstCncr = 1 if inlist(substr(dx`ii',1,3),"185")
	replace BrstCncr = 1 if inlist(substr(dx`ii',1,3),"174")
	replace IngHrn = 1 if inlist(substr(dx`ii',1,3),"550") 
	replace SpDscHrn = 1 if inlist(substr(dx`ii',1,4),"7220","7221","7222")		
}
assert PrstCncr + BrstCncr + IngHrn + SpDscHrn > 0
cou if PrstCncr + BrstCncr + IngHrn + SpDscHrn > 1
drop if PrstCncr + BrstCncr + IngHrn + SpDscHrn > 1 /* Very few simultaneous visits 
	for more than one of these four conditions - less than .01%. */
drop if mi(enrolid) | mi(plantyp) /* Can't use these claims. */
gen byte Condition = PrstCncr + 2*BrstCncr + 3*IngHrn + 4*SpDscHrn
bys enrolid Condition: egen TotOtptPay = sum(pay)
bys enrolid Condition: egen ModalPlantyp = mode(plantyp), minmode /* Plantype must be 
	constant within an enrolid condition for the reshape command. */
cou if ModalPlantyp ~= plantyp /* These differ for < 1% of claims. */
destring proc1, generate(numproc1) force
/* Using both CPT and ICD codes here (distinguished by proctyp - "*" for ICD and "1" 
or "7" for CPT/HCPCS). Note that for prostate cancer you're including all radiation 
oncology cpt codes. So there will be lots of breast cancer patients with OtptPrstSrgRd 
= 1 too. */
gen p1 = proctyp == "*" & inlist(substr(proc1,1,3),"602","603","604","605","606") 
/* May want to include ICD implantation of radioactive elements in p1: proc1 == "9227" */
gen p2 = inlist(proctyp,"1","7") & (inrange(numproc1,55801,55866) | /// 
	inrange(numproc1,52601,52640) | inrange(numproc1,53850,53852))
/* May want to include CPT radiation oncology in p2: inrange(numproc1,77261,77799). */
gen m1 = proctyp == "*" & inlist(substr(proc1,1,3),"854","857") 
gen m2 = inlist(proctyp,"1","7") & (inrange(numproc1,19180,19240) | /// 
	inrange(numproc1,19303,19307))
gen i1 = proctyp == "*" & inlist(substr(proc1,1,2),"53")
gen i2 = inlist(proctyp,"1","7") & (inrange(numproc1,49491,49525) | ///
	inrange(numproc1,49650,49659))
gen s1 = proctyp == "*" & inlist(substr(proc1,1,3),"80","81","03") 
gen s2 = inlist(proctyp,"1","7") & inrange(numproc1,22100,22899)
bys enrolid Condition: egen OtptPrstSrgRd = max(p1 | p2)
bys enrolid Condition: egen OtptMstc = max(m1 | m2)
bys enrolid Condition: egen OtptIngHrnRp = max(i1 | i2)
bys enrolid Condition: egen OtptSpSrg = max(s1 | s2)
bys enrolid Condition: egen FirstYear = min(year)	
bys enrolid Condition (year seqnum): keep if _n == 1 /* Collapse to one record per 
	person x condition. */
keep proc1 enrolid PrstCncr BrstCncr IngHrn SpDscHrn Condition TotOtptPay /// 
	OtptPrstSrgRd OtptMstc OtptIngHrnRp OtptSpSrg ModalPlantyp FirstYear
rename ModalPlantyp plantyp
save ${Data}/ccaeo_othrconds, replace


/* -------------------------------------------------------------------------------------*/
/* Part B. Convert the inpatient data into a dataset in which there is one observation  */
/* per enrolid X condition, with indicators for "intensive" treatment of each of the    */
/* four conditions and a variable recording the total amount spent on that treatment.   */
/* -------------------------------------------------------------------------------------*/

forvalues ii = 2002/2007{
	use ${Data}/`ii'/ccaei`ii', clear
	gen Keep = 0
	qui forvalues jj = 1/15{
		replace Keep = 1 if inlist(substr(dx`jj',1,3),"185","174","550") | /// 
			inlist(substr(dx`jj',1,4),"7220","7221","7222")
	}
	keep if Keep
	drop Keep
	compress
	save ${Data}/`ii'/ccaei`ii'_othrconds, replace
}

use ${Data}/2002/ccaei2002_othrconds 
forvalues ii = 2003/2007{
	append using ${Data}/`ii'/ccaei`ii'_othrconds
}

gen byte PrstCncr = 0
gen byte BrstCncr = 0
gen byte IngHrn = 0
gen byte SpDscHrn = 0
qui forvalues ii = 1/15{
	replace PrstCncr = 1 if inlist(substr(dx`ii',1,3),"185")
	replace BrstCncr = 1 if inlist(substr(dx`ii',1,3),"174")
	replace IngHrn = 1 if inlist(substr(dx`ii',1,3),"550") 
	replace SpDscHrn = 1 if inlist(substr(dx`ii',1,4),"7220","7221","7222")		
}
assert PrstCncr + BrstCncr + IngHrn + SpDscHrn > 0
cou if PrstCncr + BrstCncr + IngHrn + SpDscHrn > 1
drop if PrstCncr + BrstCncr + IngHrn + SpDscHrn > 1 /* Very few such claims - less than 
	.5%. */
drop if mi(enrolid) | mi(plantyp)
gen byte Condition = PrstCncr + 2*BrstCncr + 3*IngHrn + 4*SpDscHrn
bys enrolid Condition: egen TotInptPhysPay = sum(totpay - hosppay)
bys enrolid Condition: egen ModalPlantyp = mode(plantyp), minmode
cou if ModalPlantyp ~= plantyp /* These differ for < 1% of claims. */
aorder
destring proc1-proc15, generate(numproc1-numproc15) force
gen TempPrstSrgRd = 0
gen TempMstc = 0
gen TempIngHrnRp = 0
gen TempSpSrg = 0
/* The numproc < 10000 commands are to identify the ICD-9 procedure codes. */
forvalues ii = 1/15{
	replace TempPrstSrgRd = 1 if ///
		(numproc`ii' < 10000 & /// 	
			(inlist(substr(proc`ii',1,3),"602","603","604","605","606"))) ///
		| inrange(numproc`ii',55801,55866) ///
		| inrange(numproc`ii',52601,52640) ///
		| inrange(numproc`ii',53850,53852)
	replace TempMstc = 1 if ///
		(numproc`ii' < 10000 & inlist(substr(proc`ii',1,3),"854","857")) ///
	 	| inrange(numproc`ii',19180,19240) ///
		| inrange(numproc`ii',19303,19307)
	replace TempIngHrnRp = 1 if ///
		(numproc`ii' < 10000 & inlist(substr(proc`ii',1,2),"53")) ///
		| inrange(numproc`ii',49491,49525) ///
		| inrange(numproc`ii',49650,49659)
	replace TempSpSrg = 1 if ///
		(numproc`ii' < 10000 & inlist(substr(proc`ii',1,3),"80","81","03")) ///
		| inrange(numproc`ii',22100,22899)
}

bys enrolid Condition: egen InptPrstSrgRd = max(TempPrstSrgRd)
bys enrolid Condition: egen InptMstc = max(TempMstc)
bys enrolid Condition: egen InptIngHrnRp = max(TempIngHrnRp)
bys enrolid Condition: egen InptSpSrg = max(TempSpSrg)
bys enrolid Condition: egen FirstYear = min(year)
bys enrolid Condition (year seqnum): keep if _n == 1 /* Collapse to one record per person 
	x condition. */
keep pproc enrolid PrstCncr BrstCncr IngHrn SpDscHrn Condition TotInptPhysPay ///
	InptPrstSrgRd InptMstc InptIngHrnRp InptSpSrg ModalPlantyp FirstYear
rename ModalPlantyp plantyp
save ${Data}/ccaei_othrconds, replace


/* -------------------------------------------------------------------------------------*/
/* Part C. Merge the inpatient and outpatient data together. Create variables for total */
/* (inpatient and outpatient) expenditure, and any recorded "intensive" treatment (in   */ 
/* inpatient or outpatient data). Summarize payments for each condition, broken down by */ 
/* plan type and intensive vs unintensive treatment.  									*/
/* -------------------------------------------------------------------------------------*/


merge 1:1 enrolid Condition using ${Data}/ccaeo_othrconds
gen PrstSrgRd = InptPrstSrgRd == 1 | OtptPrstSrgRd == 1
gen Mstc = InptMstc == 1 | OtptMstc == 1
gen IngHrnRp = InptIngHrnRp == 1 | OtptIngHrnRp == 1
gen SpSrg = InptSpSrg == 1 | OtptSpSrg == 1
bys PrstCncr: sum PrstSrgRd
bys BrstCncr: sum Mstc
bys IngHrn: sum IngHrnRp
bys SpDscHrn: sum SpSrg

mvencode TotOtptPay, mv(0) override
mvencode TotInptPhysPay, mv(0) override
gen Pay = TotOtptPay + TotInptPhysPay

/* Fraction of patients treated intensively, by plan type. */
table plantyp if PrstCncr & inlist(plantyp,2,4,5,6), c(m PrstSrgRd)
table plantyp if BrstCncr & inlist(plantyp,2,4,5,6), c(m Mstc)
table plantyp if IngHrn & inlist(plantyp,2,4,5,6), c(m IngHrnRp)
table plantyp if SpDscHrn & inlist(plantyp,2,4,5,6), c(m SpSrg)

/* Payments, by intensive and unintensive treatments. */
table plantyp PrstSrgRd if PrstCncr & inlist(plantyp,2,4,5,6), c(m Pay sd Pay) /// 
	format(%9.2g) 
table plantyp Mstc if BrstCncr & inlist(plantyp,2,4,5,6), c(m Pay sd Pay) format(%9.2g) 
table plantyp IngHrnRp if IngHrn & inlist(plantyp,2,4,5,6), c(m Pay sd Pay) format(%9.2g) 
table plantyp SpSrg if SpDscHrn & inlist(plantyp,2,4,5,6), c(m Pay sd Pay) format(%9.2g) 
bys Condition: cou if inlist(plantyp,2,4,5,6)




