/* This contains various regressions run to address the points raised by referees. */

/* Referee 2: Linear probability models of treatment choice on plan type and other regressors. */

clear all
macro drop _all
version 12.1
set more off
set matsize 7500
set maxvar 7500
/* ssc install moremata */
set type double, perm
capture log close
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code

do ${Code}/DefineGlobals.do
set seed ${seed}

do ${Code}/Elixhauser.do /* This defines the ${Elixhauser} macro. */
do ${Code}/${ModelPrograms}.do
do ${Code}/${ModelFunctions}.do 
use ${Data}/${EstSample}
do ${Code}/${SetUp}.do
aaa
tab Procedure, gen(Proc)

qui forvalues ii = 1/5{
	gen regvar = Proc`ii'*100
	eststo Proc`ii'_Reg: reg regvar $CovFE $CovNL b6.plantyp
	/* reg Proc`ii' $CovFE $CovNL $CovPlantyp */
	drop regvar
}

esttab Proc1_Reg Proc2_Reg Proc3_Reg Proc4_Reg Proc5_Reg, keep(2.plantyp 4.plantyp 5.plantyp) se(%8.2f) b(%8.2f)

/* Referee 2: Physician selection. */
local cutoff 10 /* If a hospital has fewer than this number of patients in the data it will be dropped from 
	the regressions. */
/* Get point estimates. */
bys MergingUnihosp: gen NumInHosp = _N
qui forvalues ii = 1/5{
	gen regvar = Proc`ii'*100
	areg regvar $CovFE $CovNL $CovPlantyp if NumInHosp >= `cutoff', absorb(MergingUnihosp)
	/* Extract the fixed effects and use them as the dependent variable in another regression. */
	predict HospFEs`ii', d 
	eststo HospFEs`ii'_Reg: reg HospFEs`ii' $CovFE $CovNL b6.plantyp, vce(cluster MergingUnihosp)
	drop regvar
	drop HospFEs`ii'
}

esttab HospFEs1_Reg HospFEs2_Reg HospFEs3_Reg HospFEs4_Reg HospFEs5_Reg, keep(2.plantyp 4.plantyp 5.plantyp) se(%8.2f) b(%8.2f)

/* Get bootstrapped standard errors. */

local numdraws 50
mata: drawscoefs = J(15, `numdraws', .)
forvalues draw = 1/`numdraws'{
	preserve	
		di `draw'
		bsample if NumInHosp >= `cutoff' & !mi(MergingUnihosp), cluster(MergingUnihosp) idcluster(BSampleUnihosp)
		mat coef`draw' = J(3,5,.)
		qui forvalues ii = 1/5{
			gen regvar = Proc`ii'*100
			areg regvar $CovFE $CovNL $CovPlantyp if NumInHosp >= `cutoff', absorb(BSampleUnihosp)
			/* Extract the fixed effects and use them as the dependent variable in another regression. */
			predict HospFEs`ii', d 
			reg HospFEs`ii' $CovFE $CovNL b6.plantyp, vce(cluster BSampleUnihosp)
			mat coefs = e(b)
			mat coef`draw'[1,`ii'] = coefs[1,"2.plantyp"]
			mat coef`draw'[2,`ii'] = coefs[1,"4.plantyp"]
			mat coef`draw'[3,`ii'] = coefs[1,"5.plantyp"]
			drop regvar
			drop HospFEs`ii'
		}
		mata: coef`draw' = st_matrix("coef`draw'")
		mata: drawscoefs[,`draw'] = vec(coef`draw')
		mata: drawscoefs
	restore 	
}

mata: BstrappedSE = diagonal(variance(drawscoefs')):^.5
mata: rowshape(BstrappedSE, 5)'
aaa


/* Referee 2: Capitation. Stop the dataset creation process right before you remove the capitated patients. */
drop _all
use ${Data}/EstSamplePrelim1
do ${Code}/CreateDatasetPrograms.do
ProvData

/* The following block of code is the part of SelectSampleGenVars which comes before "drop if somecap == 1".*/
	/* Select the observations for estimation. Define and redefine some variables used for 
	estimation.*/
	drop if mi(plantyp)
	keep if pdx_first3 == 410
	gen dropplantyp = !inlist(plantyp, 2, 4, 5, 6)
	sum dropplantyp /* NumberCode DropPT is the mean of dropplantyp. */
	drop if dropplantyp
	drop dropplantyp
	drop if region == "5"
	gen totphyspay = totpay - hosppay
	gen thpp = totphyspay/1000			
	drop if thpp <= 0	
	assert indstry != 0
	replace indstry = 0 if mi(indstry) /* Unlike the other variables, indstry has some 
		missing values. */
	replace urban = 1 if mi(urban) /* Few are mi(urban). Set to the most common value (=1) 
		if this is the case. */
	recode age (0/39 = 1) (40/44 = 2) (45/49 = 3) (50/54 = 4) (55/59 = 5) ///
		(60/max = 6), gen(agegroup)
	
	gen ST = pdx_first4 != 4107
	gen NoExp = (Last6MthExp <= 1)*!MiL6
	gen ExpCov = 0 /* Individuals of a given sickness probably spend different amounts in 
		different plan-types, so define ExpCov expenditure relative to others in the same 
		plan type. */


bys somecap: tab plantyp
bys somecap: tab Procedure 


/* Referee 2: Look at drug-eluting vs non drug-eluting stent usage. */
gen SomeNonEluting = 0
gen SomeEluting = 0
forvalues ii = 1/10{
	replace SomeNonEluting = 1 if proc`ii' == "3606"
	replace SomeEluting = 1 if proc`ii' == "3607"
}
replace SomeNonEluting = 0 if SomeEluting == 1
gen SomeElutingOrNonEluting = SomeEluting | SomeNonEluting

reg SomeEluting $CovFE $CovNL b6.plantyp if SomeElutingOrNonEluting


/* Referee 2: What would happen if prices for angioplasty and bypass were doubled. Mimic the 
counterfactual in Part B of Counterfactuals.do, but make a change to the payments.
Instead of comparing bundled payments to the actual payments, compare bundled payments to actual
payments but with the angioplasty and bypass prices doubled. */

clear all
macro drop _all
version 12.1
set more off
set type double, perm
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
do ${Code}/DefineGlobals.do
do ${Code}/Elixhauser.do
do ${Code}/${ModelPrograms}.do
do ${Code}/${ModelFunctions}.do 
do ${Code}/${CFFunctions}.do 
set seed 100
global Name n5000t0rSomeEmrgs10pi1ci1o10d0cl1st0_06:27:35
use ${Data}/${EstSample}
qui do ${Code}/${SetUp}.do

UL = (1e8,0) /* Unrestricted max and min prices. */
cmlnPP = FE*rshpaFE + WWA*rshpaW + Ins*rshpaIns
newcmlnPP = cmlnPP
newcmlnPP[,3] = newcmlnPP[,3] + J(rows(cmlnPP),1,log(1.5)) /* Multiply payments by 2, i.e. add log(2) to log payments. */
newcmlnPP[,4] = newcmlnPP[,4] + J(rows(cmlnPP),1,log(1.5))
cmUtilNonFin = FE*rshpbFE + WWB*rshpbW
cmUtil = cmUtilNonFin + cmUtillnPX(TypebP, rshpbP, newcmlnPP, NonResp, RegNum, Year, /// 
	HHI_Thirds)
FO(cmUtil, cmlnPP, rshpSig, UL, ChFin = ., PyFin1 = .)

